Option Compare Binary
Option Explicit On
Option Strict On

Imports MvpsOrg.Dotnet.Win32.ResLib.NativeMethods
Imports System
Imports System.Drawing
Imports System.Collections.Generic

''' <summary>Provides access to system bitmaps.</summary>
Public NotInheritable Class SystemBitmaps
    Private Shared m_Manager As New ResourceManager()
    Private Shared m_Cache As New Dictionary(Of OBM_Constants, WeakReference)()

    Private Sub New()
        '
    End Sub

    Private Shared Function GetCachedItem( _
        ByVal Key As OBM_Constants _
    ) As Bitmap
        Dim Reference As WeakReference = Nothing
        If m_Cache.TryGetValue(Key, Reference) Then
            If Reference.IsAlive Then
                Return DirectCast(Reference.Target, Bitmap)
            Else
                Dim Bitmap As Bitmap = _
                    m_Manager.LoadBitmap(New IntegerResourceId(Key))
                Reference.Target = Bitmap
                Return Bitmap
            End If
        Else
            Dim Bitmap As Bitmap = _
                m_Manager.LoadBitmap(New IntegerResourceId(Key))
            m_Cache.Add(Key, New WeakReference(Bitmap))
            Return Bitmap
        End If
    End Function

    ''' <summary>
    '''   Gets a bitmap showing the mask used for the check part of
    '''   radiobutton controls.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the mask used for the check part of
    '''   radiobutton controls.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the mask used for the check part of radiobutton
    '''   controls.
    ''' </returns>
    Public Shared ReadOnly Property ButtonCorners() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_BTNCORNERS)
        End Get
    End Property

    ''' <summary>Gets a bitmap showing a window sizing grip.</summary>
    ''' <value>The bitmap showing a window sizing grip.</value>
    ''' <returns>A bitmap showing a window sizing grip.</returns>
    Public Shared ReadOnly Property ButtonSize() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_BTSIZE)
        End Get
    End Property

    ''' <summary>Gets a bitmap showing a check mark.</summary>
    ''' <value>The bitmap showing a check mark.</value>
    ''' <returns>A bitmap showing a check mark.</returns>
    Public Shared ReadOnly Property Check() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_CHECK)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the check part of checkbox and radiobutton
    '''   controls in normal, pressed and disabled state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the check part of checkbox and radiobutton
    '''   controls in normal, pressed and disabled state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the check part of checkbox and radiobutton
    '''   controls in normal, pressed and disabled state.
    ''' </returns>
    Public Shared ReadOnly Property CheckBoxes() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_CHECKBOXES)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the down arrow of a scrollbar in normal
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the down arrow of a scrollbar in normal state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the down arrow of a scrollbar in normal state.
    ''' </returns>
    Public Shared ReadOnly Property Close() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_CLOSE)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing a combobox dropdown button.
    ''' </summary>
    ''' <value>The bitmap showing a combobox dropdown button.</value>
    ''' <returns>A bitmap showing a combobox dropdown button.</returns>
    Public Shared ReadOnly Property Combo() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_COMBO)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the down arrow of a scrollbar in normal
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the down arrow of a scrollbar in normal state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the down arrow of a scrollbar in normal state.
    ''' </returns>
    Public Shared ReadOnly Property DownArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_DNARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the down arrow of a scrollbar in pressed
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the down arrow of a scrollbar in presse state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the down arrow of a scrollbar in presse state.
    ''' </returns>
    Public Shared ReadOnly Property DownArrowDown() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_DNARROWD)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the down arrow of a scrollbar in disabled
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the down arrow of a scrollbar in disabled
    '''   state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the down arrow of a scrollbar in disabled state.
    ''' </returns>
    Public Shared ReadOnly Property DownArrowDisabled() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_DNARROWI)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the left arrow of a scrollbar in normal
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the left arrow of a scrollbar in normal state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the left arrow of a scrollbar in normal state.
    ''' </returns>
    Public Shared ReadOnly Property LeftArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_LFARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the left arrow of a scrollbar in pressed
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the left arrow of a scrollbar in pressed state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the left arrow of a scrollbar in pressed state.
    ''' </returns>
    Public Shared ReadOnly Property LeftArrowDown() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_LFARROWD)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the left arrow of a scrollbar in disabled
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the left arrow of a scrollbar in disabled
    '''   state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the left arrow of a scrollbar in disabled state.
    ''' </returns>
    Public Shared ReadOnly Property LeftArrowDisabled() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_LFARROWI)
        End Get
    End Property

    ''' <summary>Gets a bitmap showing a menu popup arrow.</summary>
    ''' <value>The bitmap showing a menu popup arrow.</value>
    ''' <returns>A bitmap showing a menu popup arrow.</returns>
    Public Shared ReadOnly Property MenuArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_MNARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the close title bar button in Windows 3.11
    '''   style.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the close title bar button in Windows 3.11
    '''   style.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the close title bar button in Windows 3.11
    '''   style.
    ''' </returns>
    Public Shared ReadOnly Property OldClose() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_OLD_CLOSE)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the down arrow of a scrollbar in Windows
    '''   3.11 style.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the down arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the down arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </returns>
    Public Shared ReadOnly Property OldDownArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_OLD_DNARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the left arrow of a scrollbar in Windows
    '''   3.11 style.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the left arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the left arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </returns>
    Public Shared ReadOnly Property OldLeftArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_OLD_LFARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the reduce title bar button in Windows 3.11
    '''   style.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the reduce title bar button in Windows 3.11
    '''   style.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the reduce title bar button in Windows 3.11
    '''   style.
    ''' </returns>
    Public Shared ReadOnly Property OldReduce() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_OLD_REDUCE)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the restore title bar button in Windows
    '''   3.11 style.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the restore title bar button in Windows 3.11
    '''   style.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the restore title bar button in Windows 3.11
    '''   style.
    ''' </returns>
    Public Shared ReadOnly Property OldRestore() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_OLD_RESTORE)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the right arrow of a scrollbar in Windows
    '''   3.11 style.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the right arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the right arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </returns>
    Public Shared ReadOnly Property OldRightArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_OLD_RGARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the up arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the up arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the up arrow of a scrollbar in Windows 3.11
    '''   style.
    ''' </returns>
    Public Shared ReadOnly Property OldUpArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_OLD_UPARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the zoom title bar button in Windows 3.11
    '''   style.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the zoom title bar button in Windows 3.11
    '''   style.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the zoom title bar button in Windows 3.11 style.
    ''' </returns>
    Public Shared ReadOnly Property OldZoom() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_OLD_ZOOM)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the reduce title bar button in normal
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the reduce title bar button in normal state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the reduce title bar button in normal state.
    ''' </returns>
    Public Shared ReadOnly Property Reduce() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_REDUCE)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the reduce title bar button in pressed
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the reduce title bar button in pressed state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the reduce title bar button in pressed state.
    ''' </returns>
    Public Shared ReadOnly Property ReduceDown() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_REDUCED)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the restore title bar button in normal
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the restore title bar button in normal state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the restore title bar button in normal state.
    ''' </returns>
    Public Shared ReadOnly Property Restore() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_RESTORE)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the restore title bar button in pressed
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the restore title bar button in pressed state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the restore title bar button in pressed state.
    ''' </returns>
    Public Shared ReadOnly Property RestoreDown() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_RESTORED)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the right arrow of a scrollbar in normal
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the right arrow of a scrollbar in normal
    '''   state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the right arrow of a scrollbar in normal state.
    ''' </returns>
    Public Shared ReadOnly Property RightArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_RGARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the right arrow of a scrollbar in pressed
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the right arrow of a scrollbar in pressed
    '''   state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the right arrow of a scrollbar in pressed state.
    ''' </returns>
    Public Shared ReadOnly Property RightArrowDown() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_RGARROWD)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the right arrow of a scrollbar in disabled
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the right arrow of a scrollbar in disabled
    '''   state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the right arrow of a scrollbar in disabled
    '''   state.
    ''' </returns>
    Public Shared ReadOnly Property RightArrowDisabled() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_RGARROWI)
        End Get
    End Property

    ''' <summary>Gets a bitmap showing a window sizing grip.</summary>
    ''' <value>The bitmap showing a window sizing grip.</value>
    ''' <returns>A bitmap showing a window sizing grip.</returns>
    Public Shared ReadOnly Property Size() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_SIZE)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the up arrow of a scrollbar in normal
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the up arrow of a scrollbar in normal state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the up arrow of a scrollbar in normal state.
    ''' </returns>
    Public Shared ReadOnly Property UpArrow() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_UPARROW)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the up arrow of a scrollbar in pressed
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the up arrow of a scrollbar in pressed state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the up arrow of a scrollbar in pressed state.
    ''' </returns>
    Public Shared ReadOnly Property UpArrowDown() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_UPARROWD)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the up arrow of a scrollbar in disabled
    '''   state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the up arrow of a scrollbar in disabled state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the up arrow of a scrollbar in disabled state.
    ''' </returns>
    Public Shared ReadOnly Property UpArrowDisabled() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_UPARROWI)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the zoom title bar button in normal state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the zoom title bar button in normal state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the zoom title bar button in normal state.
    ''' </returns>
    Public Shared ReadOnly Property Zoom() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_ZOOM)
        End Get
    End Property

    ''' <summary>
    '''   Gets a bitmap showing the zoom title bar button in pressed state.
    ''' </summary>
    ''' <value>
    '''   The bitmap showing the zoom title bar button in pressed state.
    ''' </value>
    ''' <returns>
    '''   A bitmap showing the zoom title bar button in pressed state.
    ''' </returns>
    Public Shared ReadOnly Property ZoomDown() As Bitmap
        Get
            Return GetCachedItem(OBM_Constants.OBM_ZOOMD)
        End Get
    End Property
End Class